@@ -151,11 +151,9 @@ public abstract class BasePrepareStatement extends MariaDbStatement implements P
    *                             one of
    *                             <code>Statement.RETURN_GENERATED_KEYS</code>
    *                             or <code>Statement.NO_GENERATED_KEYS</code>
-   * @throws SQLException if cannot retrieve auto increment value
    */
   public BasePrepareStatement(MariaDbConnection connection, int resultSetScrollType,
-      int resultSetConcurrency, int autoGeneratedKeys)
-      throws SQLException {
+      int resultSetConcurrency, int autoGeneratedKeys) {
     super(connection, resultSetScrollType, resultSetConcurrency);
     this.noBackslashEscapes = protocol.noBackslashEscapes();
     this.useFractionalSeconds = options.useFractionalSeconds;
@@ -647,14 +645,14 @@ public abstract class BasePrepareStatement extends MariaDbStatement implements P
    * <P><B>Note:</B> You must specify the parameter's SQL type.</P>
    *
    * @param parameterIndex the first parameter is 1, the second is 2, ...
-   * @param mysqlType      the type code defined in <code> ColumnType</code>
+   * @param mariadbType      the type code defined in <code> ColumnType</code>
    * @throws SQLException                    if parameterIndex does not correspond to a parameter
    *                                         marker in the SQL statement; if a database access error
    *                                         occurs or this method is called on a closed
    *                                         <code>PreparedStatement</code>
    */
-  public void setNull(final int parameterIndex, final ColumnType mysqlType) throws SQLException {
-    setParameter(parameterIndex, new NullParameter(mysqlType));
+  public void setNull(final int parameterIndex, final ColumnType mariadbType) throws SQLException {
+    setParameter(parameterIndex, new NullParameter(mariadbType));
   }
 
   /**
@@ -1026,35 +1024,35 @@ public abstract class BasePrepareStatement extends MariaDbStatement implements P
       setString(parameterIndex, obj.toString());
     } else if (obj instanceof Clob) {
       setClob(parameterIndex, (Clob) obj);
-    } else if (LocalDateTime.class.isInstance(obj)) {
-      setTimestamp(parameterIndex, Timestamp.valueOf(LocalDateTime.class.cast(obj)));
-    } else if (Instant.class.isInstance(obj)) {
-      setTimestamp(parameterIndex, Timestamp.from(Instant.class.cast(obj)));
-    } else if (LocalDate.class.isInstance(obj)) {
-      setDate(parameterIndex, Date.valueOf(LocalDate.class.cast(obj)));
-    } else if (OffsetDateTime.class.isInstance(obj)) {
+    } else if (obj instanceof LocalDateTime) {
+      setTimestamp(parameterIndex, Timestamp.valueOf((LocalDateTime) obj));
+    } else if (obj instanceof Instant) {
+      setTimestamp(parameterIndex, Timestamp.from((Instant) obj));
+    } else if (obj instanceof LocalDate) {
+      setDate(parameterIndex, Date.valueOf((LocalDate) obj));
+    } else if (obj instanceof OffsetDateTime) {
       setParameter(parameterIndex,
           new ZonedDateTimeParameter(
-              OffsetDateTime.class.cast(obj).toZonedDateTime(),
+              ((OffsetDateTime) obj).toZonedDateTime(),
               protocol.getTimeZone().toZoneId(),
               useFractionalSeconds,
               options));
-    } else if (OffsetTime.class.isInstance(obj)) {
+    } else if (obj instanceof OffsetTime) {
       setParameter(parameterIndex,
           new OffsetTimeParameter(
-              OffsetTime.class.cast(obj),
+                  (OffsetTime) obj,
               protocol.getTimeZone().toZoneId(),
               useFractionalSeconds,
               options));
-    } else if (ZonedDateTime.class.isInstance(obj)) {
+    } else if (obj instanceof ZonedDateTime) {
       setParameter(parameterIndex,
           new ZonedDateTimeParameter(
-              ZonedDateTime.class.cast(obj),
+                  (ZonedDateTime) obj,
               protocol.getTimeZone().toZoneId(),
               useFractionalSeconds,
               options));
-    } else if (LocalTime.class.isInstance(obj)) {
-      setTime(parameterIndex, Time.valueOf(LocalTime.class.cast(obj)));
+    } else if (obj instanceof LocalTime) {
+      setTime(parameterIndex, Time.valueOf((LocalTime) obj));
     } else {
       //fallback to sending serialized object
       setParameter(parameterIndex, new SerializableParameter(obj, noBackslashEscapes));
@@ -1248,35 +1246,35 @@ public abstract class BasePrepareStatement extends MariaDbStatement implements P
       setBinaryStream(parameterIndex, (InputStream) obj, scaleOrLength);
     } else if (obj instanceof Reader) {
       setCharacterStream(parameterIndex, (Reader) obj, scaleOrLength);
-    } else if (LocalDateTime.class.isInstance(obj)) {
-      setTimestamp(parameterIndex, Timestamp.valueOf(LocalDateTime.class.cast(obj)));
-    } else if (Instant.class.isInstance(obj)) {
-      setTimestamp(parameterIndex, Timestamp.from(Instant.class.cast(obj)));
-    } else if (LocalDate.class.isInstance(obj)) {
-      setDate(parameterIndex, Date.valueOf(LocalDate.class.cast(obj)));
-    } else if (OffsetDateTime.class.isInstance(obj)) {
+    } else if (obj instanceof LocalDateTime) {
+      setTimestamp(parameterIndex, Timestamp.valueOf((LocalDateTime) obj));
+    } else if (obj instanceof Instant) {
+      setTimestamp(parameterIndex, Timestamp.from((Instant) obj));
+    } else if (obj instanceof LocalDate) {
+      setDate(parameterIndex, Date.valueOf((LocalDate) obj));
+    } else if (obj instanceof OffsetDateTime) {
       setParameter(parameterIndex,
           new ZonedDateTimeParameter(
-              OffsetDateTime.class.cast(obj).toZonedDateTime(),
+              ((OffsetDateTime) obj).toZonedDateTime(),
               protocol.getTimeZone().toZoneId(),
               useFractionalSeconds,
               options));
-    } else if (OffsetTime.class.isInstance(obj)) {
+    } else if (obj instanceof OffsetTime) {
       setParameter(parameterIndex,
           new OffsetTimeParameter(
-              OffsetTime.class.cast(obj),
+                  (OffsetTime) obj,
               protocol.getTimeZone().toZoneId(),
               useFractionalSeconds,
               options));
-    } else if (ZonedDateTime.class.isInstance(obj)) {
+    } else if (obj instanceof ZonedDateTime) {
       setParameter(parameterIndex,
           new ZonedDateTimeParameter(
-              ZonedDateTime.class.cast(obj),
+                  (ZonedDateTime) obj,
               protocol.getTimeZone().toZoneId(),
               useFractionalSeconds,
               options));
-    } else if (LocalTime.class.isInstance(obj)) {
-      setTime(parameterIndex, Time.valueOf(LocalTime.class.cast(obj)));
+    } else if (obj instanceof LocalTime) {
+      setTime(parameterIndex, Time.valueOf((LocalTime) obj));
     } else {
       throw ExceptionMapper.getSqlException(
           "Could not set parameter in setObject, could not convert: " + obj.getClass() + " to "
