@@ -58,6 +58,7 @@ import java.sql.BatchUpdateException;
 import java.sql.DatabaseMetaData;
 import java.sql.ResultSet;
 import java.sql.SQLException;
+import java.sql.SQLFeatureNotSupportedException;
 import java.sql.SQLTimeoutException;
 import java.sql.SQLWarning;
 import java.sql.Statement;
@@ -69,7 +70,6 @@ import java.util.concurrent.Future;
 import java.util.concurrent.ScheduledExecutorService;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.locks.ReentrantLock;
-import org.mariadb.jdbc.internal.com.read.dao.CmdInformation;
 import org.mariadb.jdbc.internal.com.read.dao.Results;
 import org.mariadb.jdbc.internal.com.read.resultset.SelectResultSet;
 import org.mariadb.jdbc.internal.logging.Logger;
@@ -125,10 +125,8 @@ public class MariaDbStatement implements Statement, Cloneable {
    * @param resultSetConcurrency a concurrency type; one of <code>ResultSet.CONCUR_READ_ONLY</code>
    *                             or
    *                             <code>ResultSet.CONCUR_UPDATABLE</code>
-   * @throws SQLException if cannot retrieve auto increment value
    */
-  public MariaDbStatement(MariaDbConnection connection, int resultSetScrollType,
-      int resultSetConcurrency) throws SQLException {
+  public MariaDbStatement(MariaDbConnection connection, int resultSetScrollType, int resultSetConcurrency) {
     this.protocol = connection.getProtocol();
     this.connection = connection;
     this.canUseServerTimeout = connection.canUseServerTimeout();
@@ -136,6 +134,7 @@ public class MariaDbStatement implements Statement, Cloneable {
     this.resultSetConcurrency = resultSetConcurrency;
     this.lock = this.connection.lock;
     this.options = this.protocol.getOptions();
+    this.fetchSize = this.options.defaultFetchSize;
   }
 
   /**
@@ -153,19 +152,23 @@ public class MariaDbStatement implements Statement, Cloneable {
     clone.batchQueries = new ArrayList<>();
     clone.closed = false;
     clone.warningsCleared = true;
-    clone.fetchSize = 0;
     clone.maxRows = 0;
+    clone.fetchSize = this.options.defaultFetchSize;
+
     return clone;
   }
 
   // Part of query prolog - setup timeout timer
-  protected void setTimerTask() {
+  protected void setTimerTask(boolean isBatch) {
     assert (timerTaskFuture == null);
 
     timerTaskFuture = timeoutScheduler.schedule(() -> {
       try {
         isTimedout = true;
-        protocol.cancelCurrentQuery();
+        if (!isBatch) {
+          protocol.cancelCurrentQuery();
+        }
+        protocol.interrupt();
       } catch (Throwable e) {
         //eat
       }
@@ -179,18 +182,17 @@ public class MariaDbStatement implements Statement, Cloneable {
    * <li>launch timeout timer if needed</li>
    * </ol>
    *
-   * @param forceUseOfTimer even if query timeout if possible on server using max_statement_time,
-   *                        force using timer (for batch)
+   * @param isBatch is batch
    * @throws SQLException if statement is closed
    */
-  protected void executeQueryPrologue(boolean forceUseOfTimer) throws SQLException {
+  protected void executeQueryPrologue(boolean isBatch) throws SQLException {
     executing = true;
     if (closed) {
       throw new SQLException("execute() is called on closed statement");
     }
     protocol.prolog(maxRows, protocol.getProxy() != null, connection, this);
-    if (queryTimeout != 0 && (!canUseServerTimeout || forceUseOfTimer)) {
-      setTimerTask();
+    if (queryTimeout != 0 && (!canUseServerTimeout || isBatch)) {
+      setTimerTask(isBatch);
     }
   }
 
@@ -230,6 +232,14 @@ public class MariaDbStatement implements Statement, Cloneable {
       }
     }
 
+    if (sqle.getErrorCode() == 1148 && !options.allowLocalInfile) {
+      return new SQLFeatureNotSupportedException(
+              "(conn:" + getServerThreadId() + ") Usage of LOCAL INFILE is disabled. "
+              + "To use it enable it via the connection property allowLocalInfile=true",
+              "42000",
+              1148, sqle);
+    }
+
     if (isTimedout) {
       return new SQLTimeoutException("(conn:" + getServerThreadId() + ") Query timed out", "JZ0002",
           1317, sqle);
@@ -272,15 +282,14 @@ public class MariaDbStatement implements Statement, Cloneable {
     return sqle;
   }
 
-  protected BatchUpdateException executeBatchExceptionEpilogue(SQLException initialSqle,
-      CmdInformation cmdInformation, int size) {
+  protected BatchUpdateException executeBatchExceptionEpilogue(SQLException initialSqle, int size) {
     SQLException sqle = handleFailoverAndTimeout(initialSqle);
     int[] ret;
-    if (cmdInformation == null) {
+    if (results == null || !results.commandEnd()) {
       ret = new int[size];
       Arrays.fill(ret, Statement.EXECUTE_FAILED);
     } else {
-      ret = cmdInformation.getUpdateCounts();
+      ret = results.getCmdInformation().getUpdateCounts();
     }
 
     sqle = ExceptionMapper.getException(sqle, connection, this, queryTimeout != 0);
@@ -290,34 +299,6 @@ public class MariaDbStatement implements Statement, Cloneable {
         sqle);
   }
 
-  /**
-   * Handle Exception for large batch update (return BatchUpdateException with long[].
-   *
-   * @param initialException initial exception
-   * @param cmdInformation   command return information (to indicate output that have been
-   *                         executed)
-   * @param size             initial batch length
-   * @return a BatchUpdateException
-   */
-  private BatchUpdateException executeLargeBatchExceptionEpilogue(SQLException initialException,
-      CmdInformation cmdInformation, int size) {
-    SQLException exception = handleFailoverAndTimeout(initialException);
-    long[] ret;
-    if (cmdInformation == null) {
-      ret = new long[size];
-      Arrays.fill(ret, Statement.EXECUTE_FAILED);
-    } else {
-      ret = cmdInformation.getLargeUpdateCounts();
-    }
-
-    exception = ExceptionMapper.getException(exception, connection, this, getQueryTimeout() != 0);
-    logger.error("error executing query", exception);
-
-    return new BatchUpdateException(exception.getMessage(), exception.getSQLState(),
-        exception.getErrorCode(), ret, exception);
-  }
-
-
   /**
    * Executes a query.
    *
@@ -345,7 +326,8 @@ public class MariaDbStatement implements Statement, Cloneable {
           resultSetScrollType,
           resultSetConcurrency,
           autoGeneratedKeys,
-          protocol.getAutoIncrementIncrement());
+          protocol.getAutoIncrementIncrement(),
+          sql, null);
 
       protocol.executeQuery(protocol.isMasterConnection(), results,
           getTimeoutSql(Utils.nativeSql(sql, protocol.noBackslashEscapes())));
@@ -382,7 +364,7 @@ public class MariaDbStatement implements Statement, Cloneable {
 
       executeQueryPrologue(false);
       results = new Results(this, fetchSize, false, 1, false, resultSetScrollType,
-          resultSetConcurrency, Statement.NO_GENERATED_KEYS, protocol.getAutoIncrementIncrement());
+          resultSetConcurrency, Statement.NO_GENERATED_KEYS, protocol.getAutoIncrementIncrement(), sql, null);
       protocol.executeQuery(protocol.isMasterConnection(), results,
           getTimeoutSql(Utils.nativeSql(sql, protocol.noBackslashEscapes())), charset);
       results.commandEnd();
@@ -768,11 +750,9 @@ public class MariaDbStatement implements Statement, Cloneable {
    * discarded. For maximum portability, use values greater than 256.
    *
    * @param max the new column size limit in bytes; zero means there is no limit
-   * @throws SQLException if a database access error occurs, this method is called on a closed
-   *                      <code>Statement</code> or the condition max &gt;= 0 is not satisfied
    * @see #getMaxFieldSize
    */
-  public void setMaxFieldSize(final int max) throws SQLException {
+  public void setMaxFieldSize(final int max) {
     maxFieldSize = max;
   }
 
@@ -783,11 +763,9 @@ public class MariaDbStatement implements Statement, Cloneable {
    *
    * @return the current maximum number of rows for a <code>ResultSet</code> object produced by this
    * <code>Statement</code> object; zero means there is no limit
-   * @throws SQLException if a database access error occurs or this method is called on a closed
-   *                      <code>Statement</code>
    * @see #setMaxRows
    */
-  public int getMaxRows() throws SQLException {
+  public int getMaxRows() {
     return (int) maxRows;
   }
 
@@ -842,10 +820,8 @@ public class MariaDbStatement implements Statement, Cloneable {
    * <code>PreparedStatements</code> objects will have no effect.
    *
    * @param enable <code>true</code> to enable escape processing; <code>false</code> to disable it
-   * @throws SQLException if a database access error occurs or this method is called on a closed
-   *                      <code>Statement</code>
    */
-  public void setEscapeProcessing(final boolean enable) throws SQLException {
+  public void setEscapeProcessing(final boolean enable) {
     //not handled
   }
 
@@ -958,10 +934,8 @@ public class MariaDbStatement implements Statement, Cloneable {
    * method, the method <code>getWarnings</code> will return <code>null</code> until a new warning
    * is reported for this <code>Statement</code> object.
    *
-   * @throws SQLException if a database access error occurs or this method is called on a closed
-   *                      <code>Statement</code>
    */
-  public void clearWarnings() throws SQLException {
+  public void clearWarnings() {
     warningsCleared = true;
   }
 
@@ -991,9 +965,8 @@ public class MariaDbStatement implements Statement, Cloneable {
    * Gets the connection that created this statement.
    *
    * @return the connection
-   * @throws SQLException if connection is invalid
    */
-  public MariaDbConnection getConnection() throws SQLException {
+  public MariaDbConnection getConnection() {
     return this.connection;
   }
 
@@ -1022,11 +995,9 @@ public class MariaDbStatement implements Statement, Cloneable {
    * <code>Statement</code> object.
    *
    * @return either <code>ResultSet.HOLD_CURSORS_OVER_COMMIT</code> or <code>ResultSet.CLOSE_CURSORS_AT_COMMIT</code>
-   * @throws SQLException if a database access error occurs or this method is called on a closed
-   *                      <code>Statement</code>
    * @since 1.4
    */
-  public int getResultSetHoldability() throws SQLException {
+  public int getResultSetHoldability() {
     return ResultSet.HOLD_CURSORS_OVER_COMMIT;
   }
 
@@ -1035,10 +1006,9 @@ public class MariaDbStatement implements Statement, Cloneable {
    * is closed if the method close has been called on it, or if it is automatically closed.
    *
    * @return true if this <code>Statement</code> object is closed; false if it is still open
-   * @throws SQLException if a database access error occurs
    * @since 1.6
    */
-  public boolean isClosed() throws SQLException {
+  public boolean isClosed() {
     return closed;
   }
 
@@ -1047,12 +1017,11 @@ public class MariaDbStatement implements Statement, Cloneable {
    *
    * @return <code>true</code> if the <code>Statement</code> is poolable; <code>false</code>
    *     otherwise
-   * @throws SQLException if this method is called on a closed <code>Statement</code>
    * @see Statement#setPoolable(boolean) setPoolable(boolean)
    * @since 1.6
    */
   @Override
-  public boolean isPoolable() throws SQLException {
+  public boolean isPoolable() {
     return false;
   }
 
@@ -1068,11 +1037,10 @@ public class MariaDbStatement implements Statement, Cloneable {
    *
    * @param poolable requests that the statement be pooled if true and that the statement not be
    *                 pooled if false
-   * @throws SQLException if this method is called on a closed <code>Statement</code>
    * @since 1.6
    */
   @Override
-  public void setPoolable(final boolean poolable) throws SQLException {
+  public void setPoolable(final boolean poolable) {
     // not handled
   }
 
@@ -1096,10 +1064,8 @@ public class MariaDbStatement implements Statement, Cloneable {
    *
    * @return the current result as an update count; -1 if the current result is a ResultSet object
    *     or there are no more results
-   * @throws SQLException if a database access error occurs or this method is called on a closed
-   *                      Statement
    */
-  public int getUpdateCount() throws SQLException {
+  public int getUpdateCount() {
     if (results != null && results.getCmdInformation() != null && !results.isBatch()) {
       return results.getCmdInformation().getUpdateCount();
     }
@@ -1189,12 +1155,10 @@ public class MariaDbStatement implements Statement, Cloneable {
    *
    * @return the default fetch direction for result sets generated from this <code>Statement</code>
    *     object
-   * @throws SQLException if a database access error occurs or this method is called on a closed
-   *                      <code>Statement</code>
    * @see #setFetchDirection
    * @since 1.2
    */
-  public int getFetchDirection() throws SQLException {
+  public int getFetchDirection() {
     return ResultSet.FETCH_FORWARD;
   }
 
@@ -1207,15 +1171,10 @@ public class MariaDbStatement implements Statement, Cloneable {
    * its own methods for getting and setting its own fetch direction. </p>
    *
    * @param direction the initial direction for processing rows
-   * @throws SQLException if a database access error occurs, this method is called on a closed
-   *                      <code>Statement</code> or the given direction is not one of
-   *                      <code>ResultSet.FETCH_FORWARD</code>,
-   *                      <code>ResultSet.FETCH_REVERSE</code>,
-   *                      or <code>ResultSet.FETCH_UNKNOWN</code>
    * @see #getFetchDirection
    * @since 1.2
    */
-  public void setFetchDirection(final int direction) throws SQLException {
+  public void setFetchDirection(final int direction) {
     //not implemented
   }
 
@@ -1227,11 +1186,9 @@ public class MariaDbStatement implements Statement, Cloneable {
    *
    * @return the default fetch size for result sets generated from this <code>Statement</code>
    *     object
-   * @throws SQLException if a database access error occurs or this method is called on a closed
-   *                      <code>Statement</code>
    * @see #setFetchSize
    */
-  public int getFetchSize() throws SQLException {
+  public int getFetchSize() {
     return this.fetchSize;
   }
 
@@ -1263,11 +1220,9 @@ public class MariaDbStatement implements Statement, Cloneable {
    * <code>Statement</code> object.
    *
    * @return either <code>ResultSet.CONCUR_READ_ONLY</code> or <code>ResultSet.CONCUR_UPDATABLE</code>
-   * @throws SQLException if a database access error occurs or this method is called on a closed
-   *                      <code>Statement</code>
    * @since 1.2
    */
-  public int getResultSetConcurrency() throws SQLException {
+  public int getResultSetConcurrency() {
     return resultSetConcurrency;
   }
 
@@ -1277,10 +1232,8 @@ public class MariaDbStatement implements Statement, Cloneable {
    *
    * @return one of <code>ResultSet.TYPE_FORWARD_ONLY</code>, <code>ResultSet.TYPE_SCROLL_INSENSITIVE</code>,
    *     or <code>ResultSet.TYPE_SCROLL_SENSITIVE</code>
-   * @throws SQLException if a database access error occurs or this method is called on a closed
-   *                      <code>Statement</code>
    */
-  public int getResultSetType() throws SQLException {
+  public int getResultSetType() {
     return resultSetScrollType;
   }
 
@@ -1350,13 +1303,8 @@ public class MariaDbStatement implements Statement, Cloneable {
     try {
       internalBatchExecution(size);
       return results.getCmdInformation().getUpdateCounts();
-
     } catch (SQLException initialSqlEx) {
-      if (results != null) {
-        results.commandEnd();
-        throw executeBatchExceptionEpilogue(initialSqlEx, results.getCmdInformation(), size);
-      }
-      throw executeBatchExceptionEpilogue(initialSqlEx, null, size);
+      throw executeBatchExceptionEpilogue(initialSqlEx, size);
     } finally {
       executeBatchEpilogue();
       lock.unlock();
@@ -1384,10 +1332,7 @@ public class MariaDbStatement implements Statement, Cloneable {
       return results.getCmdInformation().getLargeUpdateCounts();
 
     } catch (SQLException initialSqlEx) {
-      if (results != null) {
-        results.commandEnd();
-      }
-      throw executeLargeBatchExceptionEpilogue(initialSqlEx, results.getCmdInformation(), size);
+      throw executeBatchExceptionEpilogue(initialSqlEx, size);
     } finally {
       executeBatchEpilogue();
       lock.unlock();
@@ -1411,7 +1356,9 @@ public class MariaDbStatement implements Statement, Cloneable {
         resultSetScrollType,
         resultSetConcurrency,
         Statement.RETURN_GENERATED_KEYS,
-        protocol.getAutoIncrementIncrement());
+        protocol.getAutoIncrementIncrement(),
+        null,
+        null);
     protocol.executeBatchStmt(protocol.isMasterConnection(), results, batchQueries);
     results.commandEnd();
   }
@@ -1467,11 +1414,11 @@ public class MariaDbStatement implements Statement, Cloneable {
     return interfaceOrWrapper.isInstance(this);
   }
 
-  public void closeOnCompletion() throws SQLException {
+  public void closeOnCompletion() {
     mustCloseOnCompletion = true;
   }
 
-  public boolean isCloseOnCompletion() throws SQLException {
+  public boolean isCloseOnCompletion() {
     return mustCloseOnCompletion;
   }
 
