@@ -82,9 +82,9 @@ import java.time.ZonedDateTime;
 import java.util.Arrays;
 import java.util.TimeZone;
 import org.mariadb.jdbc.BasePrepareStatement;
+import org.mariadb.jdbc.ClientSidePreparedStatement;
 import org.mariadb.jdbc.MariaDbConnection;
-import org.mariadb.jdbc.MariaDbPreparedStatementClient;
-import org.mariadb.jdbc.MariaDbPreparedStatementServer;
+import org.mariadb.jdbc.ServerSidePreparedStatement;
 import org.mariadb.jdbc.internal.ColumnType;
 import org.mariadb.jdbc.internal.com.read.dao.Results;
 import org.mariadb.jdbc.internal.com.send.parameters.BigDecimalParameter;
@@ -130,8 +130,8 @@ public class UpdatableResultSet extends SelectResultSet {
   private ParameterHolder[] parameterHolders;
   private MariaDbConnection connection;
   private PreparedStatement refreshPreparedStatement = null;
-  private MariaDbPreparedStatementClient insertPreparedStatement = null;
-  private MariaDbPreparedStatementClient deletePreparedStatement = null;
+  private ClientSidePreparedStatement insertPreparedStatement = null;
+  private ClientSidePreparedStatement deletePreparedStatement = null;
 
 
   /**
@@ -156,7 +156,7 @@ public class UpdatableResultSet extends SelectResultSet {
 
 
   @Override
-  public int getConcurrency() throws SQLException {
+  public int getConcurrency() {
     return CONCUR_UPDATABLE;
   }
 
@@ -904,35 +904,35 @@ public class UpdatableResultSet extends SelectResultSet {
       updateBinaryStream(parameterIndex, (InputStream) obj, scaleOrLength);
     } else if (obj instanceof Reader) {
       updateCharacterStream(parameterIndex, (Reader) obj, scaleOrLength);
-    } else if (LocalDateTime.class.isInstance(obj)) {
-      updateTimestamp(parameterIndex, Timestamp.valueOf(LocalDateTime.class.cast(obj)));
-    } else if (Instant.class.isInstance(obj)) {
-      updateTimestamp(parameterIndex, Timestamp.from(Instant.class.cast(obj)));
-    } else if (LocalDate.class.isInstance(obj)) {
-      updateDate(parameterIndex, Date.valueOf(LocalDate.class.cast(obj)));
-    } else if (OffsetDateTime.class.isInstance(obj)) {
+    } else if (obj instanceof LocalDateTime) {
+      updateTimestamp(parameterIndex, Timestamp.valueOf((LocalDateTime) obj));
+    } else if (obj instanceof Instant) {
+      updateTimestamp(parameterIndex, Timestamp.from((Instant) obj));
+    } else if (obj instanceof LocalDate) {
+      updateDate(parameterIndex, Date.valueOf((LocalDate) obj));
+    } else if (obj instanceof OffsetDateTime) {
       parameterHolders[parameterIndex - 1] =
           new ZonedDateTimeParameter(
-              OffsetDateTime.class.cast(obj).toZonedDateTime(),
+              ((OffsetDateTime) obj).toZonedDateTime(),
               timeZone.toZoneId(),
               options.useFractionalSeconds,
               options);
-    } else if (OffsetTime.class.isInstance(obj)) {
+    } else if (obj instanceof OffsetTime) {
       parameterHolders[parameterIndex - 1] =
           new OffsetTimeParameter(
-              OffsetTime.class.cast(obj),
+                  (OffsetTime) obj,
               timeZone.toZoneId(),
               options.useFractionalSeconds,
               options);
-    } else if (ZonedDateTime.class.isInstance(obj)) {
+    } else if (obj instanceof ZonedDateTime) {
       parameterHolders[parameterIndex - 1] =
           new ZonedDateTimeParameter(
-              ZonedDateTime.class.cast(obj),
+                  (ZonedDateTime) obj,
               timeZone.toZoneId(),
               options.useFractionalSeconds,
               options);
-    } else if (LocalTime.class.isInstance(obj)) {
-      updateTime(parameterIndex, Time.valueOf(LocalTime.class.cast(obj)));
+    } else if (obj instanceof LocalTime) {
+      updateTime(parameterIndex, Time.valueOf((LocalTime) obj));
     } else {
       throw ExceptionMapper.getSqlException(
           "Could not set parameter in setObject, could not convert: " + obj.getClass() + " to "
@@ -1356,7 +1356,7 @@ public class UpdatableResultSet extends SelectResultSet {
       }
       updateSql.append(whereClause.toString());
 
-      MariaDbPreparedStatementClient preparedStatement = connection
+      ClientSidePreparedStatement preparedStatement = connection
           .clientPrepareStatement(updateSql.toString());
       int fieldsIndex = 0;
       int fieldsPrimaryIndex = 0;
@@ -1495,10 +1495,10 @@ public class UpdatableResultSet extends SelectResultSet {
           //Row has just been updated using updateRow() methods.
           //updateRow has changed primary key, must use the new value.
           if (isBinaryEncoded()) {
-            ((MariaDbPreparedStatementServer) refreshPreparedStatement)
+            ((ServerSidePreparedStatement) refreshPreparedStatement)
                 .setParameter(fieldsPrimaryIndex++, value);
           } else {
-            ((MariaDbPreparedStatementClient) refreshPreparedStatement)
+            ((ClientSidePreparedStatement) refreshPreparedStatement)
                 .setParameter(fieldsPrimaryIndex++, value);
           }
         } else {
@@ -1542,7 +1542,7 @@ public class UpdatableResultSet extends SelectResultSet {
   /**
    * {inheritDoc}.
    */
-  public void cancelRowUpdates() throws SQLException {
+  public void cancelRowUpdates() {
     Arrays.fill(parameterHolders, null);
     state = STATE_STANDARD;
   }
@@ -1562,7 +1562,7 @@ public class UpdatableResultSet extends SelectResultSet {
   /**
    * {inheritDoc}.
    */
-  public void moveToCurrentRow() throws SQLException {
+  public void moveToCurrentRow() {
     Arrays.fill(parameterHolders, null);
     state = STATE_STANDARD;
     setRowPointer(notInsertRowPointer);
