@@ -263,7 +263,7 @@ public class MastersSlavesListener extends AbstractMastersSlavesListener {
   }
 
   public boolean isMasterConnection() {
-    return (currentProtocol != null) ? currentProtocol.isMasterConnection() : true;
+    return (currentProtocol == null) || currentProtocol.isMasterConnection();
   }
 
   /**
@@ -777,11 +777,9 @@ public class MastersSlavesListener extends AbstractMastersSlavesListener {
    * @param killCmd is the fail due to a KILL cmd
    * @return an object to indicate if the previous Exception must be thrown, or the object resulting
    *     if a failover worked
-   * @throws Throwable if failover has not been catch
    */
-  public HandleErrorResult primaryFail(Method method, Object[] args, boolean killCmd)
-      throws Throwable {
-    boolean alreadyClosed = masterProtocol == null || !masterProtocol.isConnected();
+  public HandleErrorResult primaryFail(Method method, Object[] args, boolean killCmd, boolean isClosed) {
+    boolean alreadyClosed = masterProtocol == null || isClosed;
     boolean inTransaction = masterProtocol != null && masterProtocol.inTransaction();
 
     //in case of SocketTimeoutException due to having set socketTimeout, must force connection close
@@ -921,10 +919,10 @@ public class MastersSlavesListener extends AbstractMastersSlavesListener {
    * @param killCmd is fail due to a KILL command
    * @return an object to indicate if the previous Exception must be thrown, or the object resulting
    *     if a failover worked
-   * @throws Throwable if failover has not catch error
+   * @throws SQLException if relaunch operation fails
    */
   public HandleErrorResult secondaryFail(Method method, Object[] args, boolean killCmd)
-      throws Throwable {
+      throws SQLException {
     proxy.lock.lock();
     try {
       if (pingSecondaryProtocol(this.secondaryProtocol)) {
@@ -1006,6 +1004,18 @@ public class MastersSlavesListener extends AbstractMastersSlavesListener {
     return masterProtocol != null && masterProtocol.isConnected();
   }
 
+  /**
+   * Indicate if connection has an active transaction.
+   *
+   * @return boolean
+   */
+  public boolean inTransaction() {
+    if (masterProtocol != null) {
+      return masterProtocol.inTransaction();
+    }
+    return true;
+  }
+
   /**
    * Check master status.
    *
