@@ -82,8 +82,10 @@ import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.SQLNonTransientConnectionException;
 import java.sql.SQLTimeoutException;
+import java.sql.SQLTransientConnectionException;
 import java.sql.Statement;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.List;
 import java.util.Locale;
 import java.util.ServiceLoader;
@@ -112,6 +114,7 @@ import org.mariadb.jdbc.internal.logging.Logger;
 import org.mariadb.jdbc.internal.logging.LoggerFactory;
 import org.mariadb.jdbc.internal.util.BulkStatus;
 import org.mariadb.jdbc.internal.util.LogQueryTool;
+import org.mariadb.jdbc.internal.util.SqlStates;
 import org.mariadb.jdbc.internal.util.Utils;
 import org.mariadb.jdbc.internal.util.constant.ServerStatus;
 import org.mariadb.jdbc.internal.util.constant.StateChange;
@@ -127,6 +130,8 @@ import org.mariadb.jdbc.internal.util.scheduler.SchedulerServiceProviderHolder;
 public class AbstractQueryProtocol extends AbstractConnectProtocol implements Protocol {
 
   private static final Logger logger = LoggerFactory.getLogger(AbstractQueryProtocol.class);
+  private static final String CHECK_GALERA_STATE_QUERY = "show status like 'wsrep_local_state'";
+
   protected static ThreadPoolExecutor readScheduler = null;
   private final LogQueryTool logQuery;
   private int transactionIsolationLevel = 0;
@@ -135,6 +140,7 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
   private volatile int statementIdToRelease = -1;
   private FutureTask activeFutureTask = null;
   private boolean interrupted;
+  private final List<String> galeraAllowedStates;
 
   /**
    * Get a protocol instance.
@@ -147,6 +153,9 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
       final ReentrantLock lock) {
     super(urlParser, globalInfo, lock);
     logQuery = new LogQueryTool(options);
+    galeraAllowedStates = urlParser.getOptions().galeraAllowedState == null
+        ? Collections.emptyList() :
+        Arrays.asList(urlParser.getOptions().galeraAllowedState.split(","));
   }
 
   /**
@@ -219,6 +228,9 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
       getResult(results);
 
     } catch (SQLException sqlException) {
+      if ("70100".equals(sqlException.getSQLState()) && 1927 == sqlException.getErrorCode()) {
+        throw handleIoException(sqlException);
+      }
       throw logQuery.exceptionWithQuery(sql, sqlException, explicitClosed);
     } catch (IOException e) {
       throw handleIoException(e);
@@ -271,7 +283,7 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
         }
       } else {
         writer.startPacket(0);
-        ComQuery.sendSubCmd(writer, clientPrepareResult, parameters);
+        ComQuery.sendSubCmd(writer, clientPrepareResult, parameters, -1);
         writer.flush();
       }
       getResult(results);
@@ -308,7 +320,7 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
         }
       } else {
         writer.startPacket(0);
-        ComQuery.sendSubCmd(writer, clientPrepareResult, parameters);
+        ComQuery.sendSubCmd(writer, clientPrepareResult, parameters, queryTimeout);
         writer.flush();
       }
       getResult(results);
@@ -332,7 +344,8 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
    */
   public boolean executeBatchClient(boolean mustExecuteOnMaster, Results results,
       final ClientPrepareResult prepareResult,
-      final List<ParameterHolder[]> parametersList, boolean hasLongData) throws SQLException {
+      final List<ParameterHolder[]> parametersList, boolean hasLongData)
+      throws SQLException {
 
     //***********************************************************************************************************
     // Multiple solution for batching :
@@ -599,11 +612,11 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
           List<ParameterHolder[]> parametersList, List<String> queries, int paramCount,
           BulkStatus status,
           PrepareResult prepareResult)
-          throws SQLException, IOException {
+          throws IOException {
 
         ParameterHolder[] parameters = parametersList.get(status.sendCmdCounter);
         writer.startPacket(0);
-        ComQuery.sendSubCmd(writer, clientPrepareResult, parameters);
+        ComQuery.sendSubCmd(writer, clientPrepareResult, parameters, -1);
         writer.flush();
       }
 
@@ -737,7 +750,7 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
           List<ParameterHolder[]> parametersList, List<String> queries, int paramCount,
           BulkStatus status,
           PrepareResult prepareResult)
-          throws SQLException, IOException {
+          throws IOException {
 
         String sql = queries.get(status.sendCmdCounter);
         pos.startPacket(0);
@@ -1032,9 +1045,8 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
       }
 
       //send execute query
-      new ComStmtExecute(serverPrepareResult.getStatementId(), parameters,
-          parameterCount, serverPrepareResult.getParameterTypeHeader(), CURSOR_TYPE_NO_CURSOR)
-          .send(writer);
+      ComStmtExecute.send(writer, serverPrepareResult.getStatementId(), parameters,
+              parameterCount, serverPrepareResult.getParameterTypeHeader(), CURSOR_TYPE_NO_CURSOR);
       getResult(results);
 
     } catch (SQLException qex) {
@@ -1153,27 +1165,25 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
 
     int initialTimeout = -1;
     try {
-      initialTimeout = socket.getSoTimeout();
+      initialTimeout = this.socketTimeout;
       if (initialTimeout == 0) {
-        socket.setSoTimeout(timeout);
+        this.changeSocketSoTimeout(timeout);
       }
-      if (isMasterConnection() && urlParser.getOptions().galeraAllowedState != null) {
+      if (isMasterConnection() && !galeraAllowedStates.isEmpty()) {
         //this is a galera node.
         //checking not only that node is responding, but that galera state is allowed.
         Results results = new Results();
-        executeQuery(true, results, "show status like 'wsrep_local_state'");
+        executeQuery(true, results, CHECK_GALERA_STATE_QUERY);
         results.commandEnd();
         ResultSet rs = results.getResultSet();
-        if (rs == null || !rs.next()) {
-          return false;
-        }
-        String[] allowedValues = urlParser.getOptions().galeraAllowedState.split(",");
-        return Arrays.asList(allowedValues).contains(rs.getString(2)); //SYNC mode
+
+        return rs != null && rs.next() && galeraAllowedStates.contains(rs.getString(2));
       }
 
       return ping();
 
     } catch (SocketException socketException) {
+      logger.trace("Connection is not valid", socketException);
       connected = false;
       return false;
     } finally {
@@ -1181,16 +1191,16 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
       //set back initial socket timeout
       try {
         if (initialTimeout != -1) {
-          socket.setSoTimeout(initialTimeout);
+          this.changeSocketSoTimeout(initialTimeout);
         }
       } catch (SocketException socketException) {
+        logger.warn("Could not set socket timeout back to " + initialTimeout, socketException);
         connected = false;
         //eat
       }
     }
   }
 
-
   @Override
   public String getCatalog() throws SQLException {
 
@@ -1227,8 +1237,7 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
     lock.lock();
     try {
 
-      final SendChangeDbPacket packet = new SendChangeDbPacket(database);
-      packet.send(writer);
+      SendChangeDbPacket.send(writer, database);
       final Buffer buffer = reader.getPacket(true);
 
       if (buffer.getByteAt(0) == ERROR) {
@@ -1334,11 +1343,10 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
    * Returns the connection timeout in milliseconds.
    *
    * @return the connection timeout in milliseconds.
-   * @throws SocketException if there is an error in the underlying protocol, such as a TCP error.
    */
   @Override
-  public int getTimeout() throws SocketException {
-    return this.socket.getSoTimeout();
+  public int getTimeout() {
+    return this.socketTimeout;
   }
 
   /**
@@ -1351,7 +1359,7 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
   public void setTimeout(int timeout) throws SocketException {
     lock.lock();
     try {
-      this.socket.setSoTimeout(timeout);
+      this.changeSocketSoTimeout(timeout);
     } finally {
       lock.unlock();
     }
@@ -1552,7 +1560,6 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
     return autoIncrementIncrement;
   }
 
-
   /**
    * Read ERR_Packet.
    *
@@ -1623,12 +1630,28 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
           if (!interceptor.validate(fileName)) {
             writer.writeEmptyPacket();
             reader.getPacket(true);
-            throw new SQLException("LOCAL DATA LOCAL INFILE request to send local file named \""
+            throw new SQLException("LOAD DATA LOCAL INFILE request to send local file named \""
                 + fileName + "\" not validated by interceptor \"" + interceptor.getClass().getName()
                 + "\"");
           }
         }
 
+        if (results.getSql() == null) {
+          writer.writeEmptyPacket();
+          reader.getPacket(true);
+          throw new SQLException(
+                  "LOAD DATA LOCAL INFILE not permit in batch. file '" + fileName + "'",
+                  SqlStates.INVALID_AUTHORIZATION.getSqlState(), -1);
+
+        } else if (!Utils.validateFileName(results.getSql(), results.getParameters(), fileName)) {
+          writer.writeEmptyPacket();
+          reader.getPacket(true);
+          throw new SQLException(
+                  "LOAD DATA LOCAL INFILE asked for file '" + fileName + "' that doesn't correspond to initial query " + results.getSql()
+                          + ". Possible malicious proxy changing server answer ! Command interrupted",
+                  SqlStates.INVALID_AUTHORIZATION.getSqlState(), -1);
+        }
+
         try {
           URL url = new URL(fileName);
           is = url.openStream();
@@ -1770,7 +1793,6 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
     return serverPrepareStatementCache.put(key, serverPrepareResult);
   }
 
-
   private void cmdPrologue() throws SQLException {
 
     //load active result if any so buffer are clean for next query
@@ -1848,44 +1870,59 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
    * @param initialException initial Io error
    * @return the resulting error to return to client.
    */
-  public SQLException handleIoException(IOException initialException) {
-    boolean mustReconnect;
-    boolean driverPreventError = false;
+  public SQLException handleIoException(Exception initialException) {
+    boolean mustReconnect = options.autoReconnect;
+    boolean maxSizeError;
 
-    if (MaxAllowedPacketException.class.isInstance(initialException)) {
-      mustReconnect = ((MaxAllowedPacketException) initialException).isMustReconnect();
-      driverPreventError = !mustReconnect;
+    if (initialException instanceof MaxAllowedPacketException) {
+      maxSizeError = true;
+      if (((MaxAllowedPacketException) initialException).isMustReconnect()) {
+        mustReconnect = true;
+      } else {
+        return new SQLNonTransientConnectionException(initialException.getMessage() + getTraces(),
+                UNDEFINED_SQLSTATE.getSqlState(),
+                initialException);
+      }
     } else {
-      mustReconnect = writer.exceedMaxLength();
+      maxSizeError = writer.exceedMaxLength();
+      if (maxSizeError) {
+        mustReconnect = true;
+      }
     }
 
-    if (mustReconnect) {
+    if (mustReconnect && !explicitClosed) {
       try {
         connect();
+
+        try {
+          resetStateAfterFailover(getMaxRows(), getTransactionIsolationLevel(), getDatabase(),
+                  getAutocommit());
+
+          if (maxSizeError) {
+            return new SQLTransientConnectionException("Could not send query: query size is >= to max_allowed_packet ("
+                    + writer.getMaxAllowedPacket() + ")" + getTraces(), UNDEFINED_SQLSTATE.getSqlState(),
+                    initialException);
+          }
+
+          return new SQLNonTransientConnectionException(initialException.getMessage() + getTraces(),
+                  UNDEFINED_SQLSTATE.getSqlState(),
+                  initialException);
+
+        } catch (SQLException queryException) {
+          return new SQLNonTransientConnectionException("reconnection succeed, but resetting previous state failed",
+                  UNDEFINED_SQLSTATE.getSqlState() + getTraces(), initialException);
+        }
+
       } catch (SQLException queryException) {
         connected = false;
         return new SQLNonTransientConnectionException(initialException.getMessage()
             + "\nError during reconnection" + getTraces(), CONNECTION_EXCEPTION.getSqlState(),
             initialException);
       }
-
-      try {
-        resetStateAfterFailover(getMaxRows(), getTransactionIsolationLevel(), getDatabase(),
-            getAutocommit());
-      } catch (SQLException queryException) {
-        return new SQLException("reconnection succeed, but resetting previous state failed",
-            UNDEFINED_SQLSTATE.getSqlState() + getTraces(), initialException);
-      }
-
-      return new SQLException("Could not send query: query size is >= to max_allowed_packet ("
-          + writer.getMaxAllowedPacket() + ")" + getTraces(), UNDEFINED_SQLSTATE.getSqlState(),
-          initialException);
     }
-    if (!driverPreventError) {
-      connected = false;
-    }
-    return new SQLException(initialException.getMessage() + getTraces(),
-        driverPreventError ? UNDEFINED_SQLSTATE.getSqlState() : CONNECTION_EXCEPTION.getSqlState(),
+    connected = false;
+    return new SQLNonTransientConnectionException(initialException.getMessage() + getTraces(),
+        CONNECTION_EXCEPTION.getSqlState(),
         initialException);
 
   }
@@ -1894,6 +1931,10 @@ public class AbstractQueryProtocol extends AbstractConnectProtocol implements Pr
     this.activeFutureTask = activeFutureTask;
   }
 
+  public void interrupt() {
+    interrupted = true;
+  }
+
   public boolean isInterrupted() {
     return interrupted;
   }
