@@ -68,6 +68,8 @@ public class Options implements Cloneable {
   public boolean trustServerCertificate;
   public String serverSslCert;
   public String trustStore;
+  public String trustStoreType;
+  public String keyStoreType;
   public String trustStorePassword;
   public String keyStore;
   public String keyStorePassword;
@@ -102,8 +104,9 @@ public class Options implements Cloneable {
   public boolean createDatabaseIfNotExist;
   public String serverTimezone;
   public boolean nullCatalogMeansCurrent = true;
-  public boolean dumpQueriesOnException = true;
+  public boolean dumpQueriesOnException;
   public boolean useOldAliasMetadataBehavior;
+  public boolean useMysqlMetadata;
   public boolean allowLocalInfile = true;
   public boolean cachePrepStmts = true;
   public int prepStmtCacheSize = 250;
@@ -126,6 +129,8 @@ public class Options implements Cloneable {
   public boolean autocommit = true;
   public boolean includeInnodbStatusInDeadlockExceptions;
   public boolean includeThreadDumpInDeadlockExceptions;
+  public String servicePrincipalName;
+  public int defaultFetchSize;
 
   //logging options
   public boolean log;
@@ -154,7 +159,7 @@ public class Options implements Cloneable {
   public boolean registerJmxPool = true;
   public int poolValidMinDelay = 1000;
   public boolean useResetConnection;
-
+  public boolean useReadAheadInput = true;
 
   @Override
   public String toString() {
@@ -283,6 +288,9 @@ public class Options implements Cloneable {
     if (includeThreadDumpInDeadlockExceptions != opt.includeThreadDumpInDeadlockExceptions) {
       return false;
     }
+    if (defaultFetchSize != opt.defaultFetchSize) {
+      return false;
+    }
     if (useBulkStmts != opt.useBulkStmts) {
       return false;
     }
@@ -331,6 +339,9 @@ public class Options implements Cloneable {
     if (useResetConnection != opt.useResetConnection) {
       return false;
     }
+    if (useReadAheadInput != opt.useReadAheadInput) {
+      return false;
+    }
     if (maxPoolSize != opt.maxPoolSize) {
       return false;
     }
@@ -529,6 +540,7 @@ public class Options implements Cloneable {
     result = 31 * result + (includeInnodbStatusInDeadlockExceptions ? 1 : 0);
     result = 31 * result + (includeThreadDumpInDeadlockExceptions ? 1 : 0);
     result = 31 * result + (useBulkStmts ? 1 : 0);
+    result = 31 * result + defaultFetchSize;
     result = 31 * result + (disableSslHostnameVerification ? 1 : 0);
     result = 31 * result + (log ? 1 : 0);
     result = 31 * result + (profileSql ? 1 : 0);
@@ -546,6 +558,7 @@ public class Options implements Cloneable {
     result = 31 * result + (pool ? 1 : 0);
     result = 31 * result + (registerJmxPool ? 1 : 0);
     result = 31 * result + (useResetConnection ? 1 : 0);
+    result = 31 * result + (useReadAheadInput ? 1 : 0);
     result = 31 * result + (staticGlobal ? 1 : 0);
     result = 31 * result + (poolName != null ? poolName.hashCode() : 0);
     result = 31 * result + (galeraAllowedState != null ? galeraAllowedState.hashCode() : 0);
