@@ -67,6 +67,7 @@ import java.util.concurrent.locks.ReentrantLock;
 import java.util.regex.Pattern;
 import javax.net.SocketFactory;
 import org.mariadb.jdbc.UrlParser;
+import org.mariadb.jdbc.internal.com.send.parameters.ParameterHolder;
 import org.mariadb.jdbc.internal.failover.FailoverProxy;
 import org.mariadb.jdbc.internal.failover.impl.AuroraListener;
 import org.mariadb.jdbc.internal.failover.impl.MastersFailoverListener;
@@ -165,7 +166,7 @@ public class Utils {
   /**
    * Encrypts a password.
    *
-   * <p>protocol for authentication is like this: 1. mysql server sends a random array of bytes (the
+   * <p>protocol for authentication is like this: 1. Server sends a random array of bytes (the
    * seed) 2. client makes a sha1 digest of the password 3. client hashes the output of 2 4. client
    * digests the seed 5. client updates the digest with the output from 3 6. an xor of the output of
    * 5 and 2 is sent to server 7. server does the same thing and verifies that the scrambled
@@ -548,7 +549,7 @@ public class Utils {
                 MastersSlavesProtocol.class.getClassLoader(),
                 new Class[]{Protocol.class},
                 new FailoverProxy(new MastersSlavesListener(urlParser, globalInfo), lock)));
-      case FAILOVER:
+      case LOADBALANCE:
       case SEQUENTIAL:
         return getProxyLoggingIfNeeded(urlParser, (Protocol) Proxy.newProxyInstance(
             MasterProtocol.class.getClassLoader(),
@@ -600,7 +601,7 @@ public class Utils {
    * @throws IOException if connection error occur
    */
   public static Socket createSocket(UrlParser urlParser, String host) throws IOException {
-    return (Socket) socketHandler.apply(urlParser, host);
+    return socketHandler.apply(urlParser, host);
   }
 
   /**
@@ -759,6 +760,29 @@ public class Utils {
     return (bytes != null) ? getHex(bytes) : "";
   }
 
+  /**
+   * Convert int value to hexadecimal String.
+   *
+   * @param value value to transform
+   * @return Hexadecimal String value of integer.
+   */
+  public static String intToHexString(final int value) {
+    final StringBuilder hex = new StringBuilder(8);
+    int offset = 24;
+    byte b;
+    boolean nullEnd = false;
+    while (offset >= 0) {
+      b = (byte) (value >> offset);
+      offset -= 8;
+      if (b != 0 || nullEnd) {
+        nullEnd = true;
+        hex.append(hexArray[(b & 0xF0) >> 4])
+                .append(hexArray[(b & 0x0F)]);
+      }
+    }
+    return hex.toString();
+  }
+
   /**
    * Parse the option "sessionVariable" to ensure having no injection. semi-column not in string
    * will be replaced by comma.
@@ -905,11 +929,37 @@ public class Utils {
     }
   }
 
-
   private enum Parse {
     Normal,
     String, /* inside string */
     Quote,
     Escape /* found backslash */
   }
+
+  /**
+   * Validate that file name correspond to send query.
+   *
+   * @param sql         sql command
+   * @param parameters  sql parameter
+   * @param fileName    server file name
+   * @return true if correspond
+   */
+  public static boolean validateFileName(String sql, ParameterHolder[] parameters, String fileName) {
+    Pattern pattern = Pattern.compile(
+            "^(\\s*\\/\\*([^\\*]|\\*[^\\/])*\\*\\/)*\\s*LOAD\\s+DATA\\s+((LOW_PRIORITY|CONCURRENT)\\s+)?LOCAL\\s+INFILE\\s+'" + fileName + "'",
+            Pattern.CASE_INSENSITIVE);
+    if (pattern.matcher(sql).find()) {
+      return true;
+    }
+
+    if (parameters != null) {
+      pattern = Pattern.compile(
+              "^(\\s*\\/\\*([^\\*]|\\*[^\\/])*\\*\\/)*\\s*LOAD\\s+DATA\\s+((LOW_PRIORITY|CONCURRENT)\\s+)?LOCAL\\s+INFILE\\s+\\?",
+              Pattern.CASE_INSENSITIVE);
+      if (pattern.matcher(sql).find() && parameters.length > 0) {
+        return parameters[0].toString().toLowerCase().equals("'" + fileName.toLowerCase() + "'");
+      }
+    }
+    return false;
+  }
 }
