@@ -58,6 +58,7 @@ import static org.junit.Assert.fail;
 
 import com.sun.jna.Platform;
 import java.io.IOException;
+import java.lang.Thread.State;
 import java.lang.reflect.Method;
 import java.net.InetAddress;
 import java.net.UnknownHostException;
@@ -67,10 +68,12 @@ import java.sql.DriverManager;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
+import java.sql.SQLNonTransientConnectionException;
 import java.sql.Statement;
 import java.text.DecimalFormat;
 import java.text.NumberFormat;
 import java.util.HashSet;
+import java.util.Iterator;
 import java.util.Map;
 import java.util.Properties;
 import java.util.Random;
@@ -142,6 +145,14 @@ public class BaseTest {
           ResultSet rs = preparedStatement.executeQuery();
           assertTrue(rs.next());
           assertEquals(randInt, rs.getInt(1));
+        } catch (SQLNonTransientConnectionException connFail) {
+          connFail.printStackTrace();
+          try {
+              beforeClassBaseTest();
+            } catch (SQLException e) {
+              System.out.println("ERROR reconnecting");
+              e.printStackTrace();
+            }
         } catch (Exception e) {
           e.printStackTrace();
           fail("Prepare after test fail for " + description.getClassName() + "." + description
@@ -179,7 +190,7 @@ public class BaseTest {
   @BeforeClass()
   public static void beforeClassBaseTest() throws SQLException {
     String url = System.getProperty("dbUrl", mDefUrl);
-    runLongTest = Boolean.getBoolean(System.getProperty("runLongTest", "false"));
+    runLongTest = Boolean.parseBoolean(System.getProperty("runLongTest", "false"));
     testSingleHost = Boolean.parseBoolean(System.getProperty("testSingleHost", "true"));
 
     if (testSingleHost) {
@@ -240,12 +251,10 @@ public class BaseTest {
   private static void setUri() {
     connU = "jdbc:mariadb://" + ((hostname == null) ? "localhost" : hostname) + ":" + port + "/"
         + ((database == null) ? "" : database);
-    connUri = connU + "?user=" + username
-        + (password != null && !"".equals(password) ? "&password=" + password : "")
-        + (parameters != null ? "&" + parameters : "");
-    connDnsUri = "jdbc:mariadb://mariadb.example.com:" + port + "/" + database + "?user=" + username
-        + (password != null && !"".equals(password) ? "&password=" + password : "")
-        + (parameters != null ? "&" + parameters : "");
+    connUri = connU + "?" + parameters
+        + (password != null && !"".equals(password) ? "&password=" + password : "");
+    connDnsUri = "jdbc:mariadb://mariadb.example.com:" + port + "/" + database + "?" + parameters
+        + (password != null && !"".equals(password) ? "&password=" + password : "");
   }
 
   /**
@@ -303,6 +312,15 @@ public class BaseTest {
         e.printStackTrace();
       }
     }
+    Iterator<Thread> it = Thread.getAllStackTraces().keySet().iterator();
+    Thread thread;
+
+    while (it.hasNext()) {
+      thread = it.next();
+      if (thread.getName().contains("MariaDb-bulk-")) {
+        assertEquals(State.WAITING, thread.getState());
+      }
+    }
   }
 
   // common function for logging information
@@ -355,9 +373,7 @@ public class BaseTest {
       Statement stmt = sharedConnection.createStatement();
       stmt.execute("drop view if exists " + viewName);
       stmt.execute("create view " + viewName + " AS (" + tableColumns + ") ");
-      if (!tempViewList.contains(viewName)) {
-        tempViewList.add(viewName);
-      }
+      tempViewList.add(viewName);
     }
   }
 
@@ -373,9 +389,7 @@ public class BaseTest {
       Statement stmt = sharedConnection.createStatement();
       stmt.execute("drop procedure IF EXISTS " + name);
       stmt.execute("create  procedure " + name + body);
-      if (!tempProcedureList.contains(name)) {
-        tempProcedureList.add(name);
-      }
+      tempProcedureList.add(name);
     }
   }
 
@@ -391,9 +405,7 @@ public class BaseTest {
       Statement stmt = sharedConnection.createStatement();
       stmt.execute("drop function IF EXISTS " + name);
       stmt.execute("create function " + name + body);
-      if (!tempFunctionList.contains(name)) {
-        tempFunctionList.add(name);
-      }
+      tempFunctionList.add(name);
     }
   }
 
@@ -676,21 +688,13 @@ public class BaseTest {
     assertTrue(rs.next());
     long maxAllowedPacket = rs.getLong(1);
 
-    rs = st.executeQuery("select @@innodb_log_file_size");
-    assertTrue(rs.next());
-    long innodbLogFileSize = rs.getLong(1);
-
     if (maxAllowedPacket < 8 * 1024 * 1024L) {
 
       System.out.println(
           "test '" + testName + "' skipped  due to server variable max_allowed_packet < 8M");
       return false;
     }
-    if (innodbLogFileSize < 80 * 1024 * 1024L) {
-      System.out.println(
-          "test '" + testName + "' skipped  due to server variable innodb_log_file_size < 80M");
-      return false;
-    }
+
     return true;
   }
 
@@ -720,10 +724,6 @@ public class BaseTest {
     assertTrue(rs.next());
     long maxAllowedPacket = rs.getLong(1);
 
-    rs = st.executeQuery("select @@innodb_log_file_size");
-    assertTrue(rs.next());
-    long innodbLogFileSize = rs.getLong(1);
-
     if (maxAllowedPacket < 20 * 1024 * 1024L) {
 
       if (displayMessage) {
@@ -732,13 +732,7 @@ public class BaseTest {
       }
       return false;
     }
-    if (innodbLogFileSize < 200 * 1024 * 1024L) {
-      if (displayMessage) {
-        System.out.println(
-            "test '" + testName + "' skipped  due to server variable innodb_log_file_size < 200M");
-      }
-      return false;
-    }
+
     return true;
   }
 
@@ -768,10 +762,6 @@ public class BaseTest {
     assertTrue(rs.next());
     long maxAllowedPacket = rs.getLong(1);
 
-    rs = st.executeQuery("select @@innodb_log_file_size");
-    assertTrue(rs.next());
-    long innodbLogFileSize = rs.getLong(1);
-
     if (maxAllowedPacket < 40 * 1024 * 1024L) {
       if (displayMsg) {
         System.out.println(
@@ -779,13 +769,6 @@ public class BaseTest {
       }
       return false;
     }
-    if (innodbLogFileSize < 400 * 1024 * 1024L) {
-      if (displayMsg) {
-        System.out.println(
-            "test '" + testName + "' skipped  due to server variable innodb_log_file_size < 400M");
-      }
-      return false;
-    }
 
     return true;
   }
