@@ -114,6 +114,10 @@ public class DriverTest extends BaseTest {
         "engine=innodb");
     createTable("Drivert4", "id int not null primary key auto_increment, test varchar(20)",
         "engine=innodb");
+    createTable("Drivert5", "id int not null primary key auto_increment, test varchar(20)",
+            "engine=innodb");
+    createTable("Drivert6", "id int not null primary key auto_increment, test varchar(20)",
+            "engine=innodb");
     createTable("test_float", "id int not null primary key auto_increment, a float");
     createTable("test_big_autoinc2",
         "id int not null primary key auto_increment, test varchar(10)");
@@ -477,18 +481,75 @@ public class DriverTest extends BaseTest {
     stmt.executeUpdate("INSERT INTO Drivert4 (test) values('hej6')");
     sharedConnection.commit();
     ResultSet rs = stmt.executeQuery("SELECT * FROM Drivert4");
-    assertEquals(true, rs.next());
+    assertTrue(rs.next());
     assertEquals("hej1", rs.getString(2));
-    assertEquals(true, rs.next());
+    assertTrue(rs.next());
     assertEquals("hej2", rs.getString(2));
-    assertEquals(true, rs.next());
+    assertTrue(rs.next());
     assertEquals("hej5", rs.getString(2));
-    assertEquals(true, rs.next());
+    assertTrue(rs.next());
     assertEquals("hej6", rs.getString(2));
-    assertEquals(false, rs.next());
+    assertFalse(rs.next());
     sharedConnection.setAutoCommit(true);
   }
 
+  @Test
+  public void savepointUnname() throws SQLException {
+    Statement stmt = sharedConnection.createStatement();
+    sharedConnection.setAutoCommit(false);
+    stmt.executeUpdate("INSERT INTO Drivert5 (test) values('hej1')");
+    stmt.executeUpdate("INSERT INTO Drivert5 (test) values('hej2')");
+    Savepoint savepoint = sharedConnection.setSavepoint();
+    stmt.executeUpdate("INSERT INTO Drivert5 (test)  values('hej3')");
+    stmt.executeUpdate("INSERT INTO Drivert5 (test) values('hej4')");
+    sharedConnection.rollback(savepoint);
+    stmt.executeUpdate("INSERT INTO Drivert5 (test) values('hej5')");
+    stmt.executeUpdate("INSERT INTO Drivert5 (test) values('hej6')");
+    sharedConnection.commit();
+    ResultSet rs = stmt.executeQuery("SELECT * FROM Drivert5");
+    assertTrue(rs.next());
+    assertEquals("hej1", rs.getString(2));
+    assertTrue(rs.next());
+    assertEquals("hej2", rs.getString(2));
+    assertTrue(rs.next());
+    assertEquals("hej5", rs.getString(2));
+    assertTrue(rs.next());
+    assertEquals("hej6", rs.getString(2));
+    assertFalse(rs.next());
+    sharedConnection.setAutoCommit(true);
+  }
+
+  @Test
+  public void releaseSavepoint() throws SQLException {
+    Statement stmt = sharedConnection.createStatement();
+    sharedConnection.setAutoCommit(false);
+    stmt.executeUpdate("INSERT INTO Drivert6 (test) values('hej1')");
+    stmt.executeUpdate("INSERT INTO Drivert6 (test) values('hej2')");
+    Savepoint savepoint = sharedConnection.setSavepoint();
+    stmt.executeUpdate("INSERT INTO Drivert6 (test)  values('hej3')");
+    stmt.executeUpdate("INSERT INTO Drivert6 (test) values('hej4')");
+    sharedConnection.releaseSavepoint(savepoint);
+    stmt.executeUpdate("INSERT INTO Drivert6 (test) values('hej5')");
+    stmt.executeUpdate("INSERT INTO Drivert6 (test) values('hej6')");
+    sharedConnection.commit();
+    ResultSet rs = stmt.executeQuery("SELECT * FROM Drivert6");
+    assertTrue(rs.next());
+    assertEquals("hej1", rs.getString(2));
+    assertTrue(rs.next());
+    assertEquals("hej2", rs.getString(2));
+    assertTrue(rs.next());
+    assertEquals("hej3", rs.getString(2));
+    assertTrue(rs.next());
+    assertEquals("hej4", rs.getString(2));
+    assertTrue(rs.next());
+    assertEquals("hej5", rs.getString(2));
+    assertTrue(rs.next());
+    assertEquals("hej6", rs.getString(2));
+    assertFalse(rs.next());
+    sharedConnection.setAutoCommit(true);
+  }
+
+
   @Test
   public void isolationLevel() throws SQLException {
     try (Connection connection = setConnection()) {
@@ -505,6 +566,18 @@ public class DriverTest extends BaseTest {
     }
   }
 
+  @Test
+  public void wrongIsolationLevel() throws SQLException {
+    try (Connection connection = setConnection()) {
+      try {
+        connection.setTransactionIsolation(10_000);
+        fail();
+      } catch (SQLException e) {
+        assertTrue(e.getMessage().contains("Unsupported transaction isolation level"));
+      }
+    }
+  }
+
   @Test
   public void isValidTest() throws SQLException {
     assertEquals(true, sharedConnection.isValid(0));
@@ -638,6 +711,7 @@ public class DriverTest extends BaseTest {
   }
 
   @Test
+  @SuppressWarnings("deprecation")
   public void connectFailover() throws SQLException {
     Assume.assumeTrue(hostname != null);
     String hosts = hostname + ":" + port + "," + hostname + ":" + (port + 1);
@@ -1143,14 +1217,14 @@ public class DriverTest extends BaseTest {
   @Test
   public void dumpQueryOnSyntaxException() {
     String syntacticallyWrongQuery = "banana";
-    try {
-      Statement st = sharedConnection.createStatement();
+    try (Connection connection = setConnection("&dumpQueriesOnException")) {
+      Statement st = connection.createStatement();
       st.execute(syntacticallyWrongQuery);
     } catch (SQLException sqle) {
       assertTrue(sqle.getCause().getMessage().contains("Query is: " + syntacticallyWrongQuery));
     }
   }
-
+  
   /* Check that query contains SQL statement, if dumpQueryOnException is true */
   @Test
   public void dumpQueryOnException() throws Exception {
@@ -1166,6 +1240,18 @@ public class DriverTest extends BaseTest {
     }
   }
 
+  /* Check that query does not contains SQL statement by default */
+  @Test
+  public void shouldNotDumpQueryOnExceptionByDefault() {
+    String selectFromNonExistingTable = "select * from mango";
+    try {
+      Statement st = sharedConnection.createStatement();
+      st.execute(selectFromNonExistingTable);
+    } catch (SQLException sqle) {
+      assertFalse(sqle.getCause().getMessage().contains("Query is: " + selectFromNonExistingTable));
+    }
+  }
+
   /* CONJ-14
    * getUpdateCount(), getResultSet() should indicate "no more results" with
    * (getUpdateCount() == -1 && getResultSet() == null)
@@ -1334,7 +1420,7 @@ public class DriverTest extends BaseTest {
   @Test
   public void localSocket() throws Exception {
     requireMinimumVersion(5, 1);
-    Assume.assumeTrue(System.getenv("TRAVIS") == null);
+    Assume.assumeTrue(System.getenv("TRAVIS") == null && System.getenv("DOCKER_SOCKET") == null);
     Assume.assumeTrue(isLocalConnection("localSocket"));
 
     Statement st = sharedConnection.createStatement();
@@ -1349,10 +1435,19 @@ public class DriverTest extends BaseTest {
     }
 
     String path = rs.getString(2);
-    try (Connection connection = setConnection("&localSocket=" + path + "&profileSql=true")) {
+    st.execute("CREATE USER testSocket@'" + ((hostname == null) ? "localhost" : hostname) + "'  IDENTIFIED VIA unix_socket");
+    try {
+      st.execute("INSTALL SONAME 'auth_socket'");
+    } catch (SQLException e) {
+      //dismiss, can already be installed
+    }
+    String connString = connU + "?user=testSocket&localSocket=" + path + "&profileSql=true";
+    System.out.println(connString);
+    try (Connection connection = openConnection(connString, null)) {
       rs = connection.createStatement().executeQuery("select 1");
       assertTrue(rs.next());
     }
+    st.execute("DROP user testSocket@'" + ((hostname == null) ? "localhost" : hostname) + "'");
   }
 
   @Test
@@ -1370,12 +1465,16 @@ public class DriverTest extends BaseTest {
       return;  // skip test on non-Windows
     }
 
-    rs = st.executeQuery("select @@shared_memory,@@shared_memory_base_name");
-    if (!rs.next()) {
-      return;
-    }
+    try {
+      rs = st.executeQuery("select @@shared_memory,@@shared_memory_base_name");
+      if (!rs.next()) {
+        return;
+      }
 
-    if (!rs.getString(1).equals("1")) {
+      if (!rs.getString(1).equals("1")) {
+        return;
+      }
+    } catch (SQLException e) {
       return;
     }
 
@@ -1578,5 +1677,16 @@ public class DriverTest extends BaseTest {
     }
   }
 
-
+  @Test
+  public void databaseType() throws SQLException {
+    Assume.assumeTrue(System.getenv("MAXSCALE_VERSION") == null);
+    Assume.assumeTrue(System.getenv("TRAVIS") != null);
+    boolean isMysql = System.getenv("AURORA") != null || System.getenv("DB").contains("mysql");
+    assertEquals(isMysql ? "MySQL" : "MariaDB", sharedConnection.getMetaData().getDatabaseProductName());
+    if (!isMysql) {
+      try (Connection connection = setConnection("&useMysqlMetadata=true")) {
+        assertEquals("MySQL", connection.getMetaData().getDatabaseProductName());
+      }
+    }
+  }
 }
