@@ -230,6 +230,7 @@ public class MariaDbPoolDataSourceTest extends BaseTest {
 
   @Test
   public void testResetTransactionIsolation() throws SQLException {
+    Assume.assumeFalse(sharedIsAurora());
     try (MariaDbPoolDataSource pool = new MariaDbPoolDataSource(connUri + "&maxPoolSize=1")) {
 
       try (Connection connection = pool.getConnection()) {
@@ -426,9 +427,7 @@ public class MariaDbPoolDataSourceTest extends BaseTest {
           ResultSet rs = stmt.executeQuery("SELECT CONNECTION_ID()");
           rs.next();
           Integer connectionId = rs.getInt(1);
-          if (!threadIds.contains(connectionId)) {
-            threadIds.add(connectionId);
-          }
+          threadIds.add(connectionId);
           stmt.execute("SELECT * FROM mysql.user");
 
         } catch (SQLException e) {
@@ -470,6 +469,7 @@ public class MariaDbPoolDataSourceTest extends BaseTest {
         connection.isValid(10_000);
       }
 
+      Thread.sleep(500);
       assertTrue(getCurrentConnections() > initialConnection);
     }
     Thread.sleep(500); //ensure that previous close are effective
